package org.myhowto.test;

import java.util.*;
import java.io.*;
import java.util.zip.*;
import java.util.regex.*;
import org.apache.commons.lang.StringUtils;

public final class StringReplaceTest {

	private static Pattern pattern = Pattern.compile("\"", Pattern.LITERAL);

	public static void main(final String[] args) throws Exception {
		if (args.length != 2) {
			System.err.println("Usage: StringReplaceTest replace|replaceAll|pattern|pureJava|commons all|smart");
			System.exit(1);
		}
		String method = args[0];
		boolean skipLookup = "all".equals(args[1]);
		long startTime = System.currentTimeMillis();
		Runtime rt = Runtime.getRuntime();
		long initialUsedMem = rt.totalMemory() - rt.freeMemory();
		final String sample1 = "abc\"def\"\"gih";
		final String sample2 = "abcdefgih";

		String[] results;

		if ("replace".equals(method)) {
			results = testReplace(sample1, sample2, skipLookup);
		} else if ("replaceAll".equals(method)) {
			results = testReplaceAll(sample1, sample2, skipLookup);
		} else if ("pattern".equals(method)) {
			results = testPattern(sample1, sample2, skipLookup);
		} else if ("pureJava".equals(method)) {
			results = testJavaImpl(sample1, sample2, skipLookup);
		} else if ("commons".equals(method)) {
			results = testStringUtils(sample1, sample2, skipLookup);
		} else {
			throw new RuntimeException("unknown method: " + method);
		}

		long usedMem = rt.totalMemory() - rt.freeMemory() - initialUsedMem;
		System.out.println("Time: " + (System.currentTimeMillis() - startTime) + " ms");
		System.out.println("Memory used: " + usedMem + " byte(s)");
	}

	private static String[] testReplace(String sample1, String sample2, boolean mask) {
		String[] results = new String[100000];
		for(int i = 0; i < results.length; i++) {
			String sample = (i % 2 == 0 ? sample1 : sample2);
			if (mask || sample.indexOf('\"') > -1) {
				results[i] = sample.replace("\"", "\"\"");
			} else {
				results[i] = sample;
			}
		}
		return results;
	}

	private static String[] testReplaceAll(String sample1, String sample2, boolean mask) {		
		String[] results = new String[100000];
		for(int i = 0; i < results.length; i++) {
			String sample = (i % 2 == 0 ? sample1 : sample2);
			if (mask || sample.indexOf('\"') > -1) {
				results[i] = sample.replaceAll("\"", "\"\"");
			} else {
				results[i] = sample;
			}
		}
		return results;
	}

	private static String[] testPattern(String sample1, String sample2, boolean mask) {
		String[] results = new String[100000];
		for(int i = 0; i < results.length; i++) {
			String sample = (i % 2 == 0 ? sample1 : sample2);
			if (mask || sample.indexOf('\"') > -1) {
				Matcher m = pattern.matcher(sample);
				results[i] = m.replaceAll("\"\"");
			} else {
				results[i] = sample;
			}
		}
		return results;
	}

	private static String[] testJavaImpl(String sample1, String sample2, boolean mask) {
		String[] results = new String[100000];
		for(int i = 0; i < results.length; i++) {
			String sample = (i % 2 == 0 ? sample1 : sample2);
			if (mask || sample.indexOf('\"') > -1) {
				int pos = 0;
				StringBuffer sb = new StringBuffer(sample);
				while((pos = sb.indexOf("\"", pos)) != -1) {
					sb.insert(pos + 1, '\"');
					pos += 2;
				}
				results[i] = sb.toString();
				
			} else {
				results[i] = sample;
			}
		}
		return results;
	}

	private static String[] testStringUtils(String sample1, String sample2, boolean mask) {
		String[] results = new String[100000];
		for(int i = 0; i < results.length; i++) {
			String sample = (i % 2 == 0 ? sample1 : sample2);
			if (mask || sample.indexOf('\"') > -1) {
				results[i] = StringUtils.replace(sample, "\"", "\"\"");
			} else {
				results[i] = sample;
			}
		}
		return results;
	}

}
